-----------------------------------------------------------------------------------------------
-- Client Lua Script for SteerClear
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SteerClear Module Definition
-----------------------------------------------------------------------------------------------
local SteerClear = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SteerClear:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    self.backups = {}
	self.steerFound = false
	
	self.configs = {}
	
	self.mapping = {
		LimitedActionSet1 = '1',
		LimitedActionSet2 = '2'
	}

    return o
end

function SteerClear:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SteerClear OnLoad
-----------------------------------------------------------------------------------------------
function SteerClear:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SteerClear.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SteerClear OnDocLoaded
-----------------------------------------------------------------------------------------------
function SteerClear:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SteerClearForm", nil, self)
		self.alert = Apollo.LoadForm(self.xmlDoc, "Alert", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
			
		if self.configs ~= nil then
		
			local buttons = {"BindOn1", "BindOn2", "BindOff1", "BindOff2", "Killswitch"}
			for button = 1, #buttons do
				local name = buttons[button]
				if self.configs[name] ~= nil then
					self.wndMain:FindChild(name):SetText(self.configs[name].name or self.configs[name].keybind or 'NO_NAME')
				end
			end
		else
			self.configs = {}
		end
		
		if self.configs.enabled == nil then
			self.configs.enabled = true
		end
		
		self:SetAlertSprite()
		
		self.alert:Show(true, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)


		-- Do additional Addon initialization here
		--Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self)
		Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self)
		Apollo.RegisterSlashCommand("steerclear", "OnSteerClearOptions", self)
		Apollo.RegisterSlashCommand("sc", "OnSteerClearOptions", self)
		
		if Apollo.GetAddon("Steer") ~= nil then
			self.steerFound = true
			
			local steer = Apollo.GetAddon("Steer")
			
			steer.oldLock = steer.Lock
			steer.Lock = function ()
				self:UnClear()
				steer.oldLock(steer)
			end
			
			steer.oldUnLock = steer.UnLock
			steer.UnLock = function ()
				self:Clear()
				steer.oldUnLock(steer)
			end
			
			local keys = GameLib.GetKeyBindings()
			for i, key in pairs(keys) do
			
				if key["strAction"] == "LimitedActionSet1" or key["strAction"] == "LimitedActionSet2" then
					local action = key["strAction"]
					local inputs = key["arInputs"]
					self.backups[action] = inputs[1]
				end
			end
			ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SteerClear v1.1.1 loaded!  /sc /steerclear", "")
		else
			ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SteerClear:  Steer could not be found.  To make use of this addon, you must install Steer by Cassitel as well.", "")
		end
	end
end

-----------------------------------------------------------------------------------------------
-- SteerClear Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

function SteerClear:SetAlertSprite()
	if self.configs.enabled then
		self.alert:SetSprite('Crafting_CoordSprites:sprCoord_SmallCircle_Green')
	else
		self.alert:SetSprite('Crafting_CoordSprites:sprCoord_SmallCircle_Red')
	end
end

function SteerClear:UnClear()
	self:Perform(false)
end

function SteerClear:Clear()
	self:Perform(true)
end

function SteerClear:Perform(clear)
	if self.configs.enabled then
		local keys = GameLib.GetKeyBindings()
		local mapping = "Bind" .. (clear and 'Off' or 'On')
			
		for i, key in pairs(keys) do
			
			if key["strAction"] == "LimitedActionSet1" or key["strAction"] == "LimitedActionSet2" then
				local action = key["strAction"]
				local inputs = key["arInputs"]
				local thisMapping = mapping .. self.mapping[key['strAction']]
				
				local config = self.configs[thisMapping]
				if config ~= nil and config.input ~= nil then
					inputs[1] = config.input
				else
					inputs[1] = self:emptyInput()
				end
			end
		end
		GameLib.SetKeyBindings(keys)
	end
end

function SteerClear:emptyInput()
	return {
		eDevice = 0,
		eModifier = 0,
		nCode = 0
	}
end

function SteerClear:OnSystemKeyDown(key)
	if self.wndMain:IsShown() then
		self:OnKeyDownForSystem(key)
		
	elseif self.configs["Killswitch"] and key == self.configs['Killswitch'].keybind then
		self.configs.enabled = not self.configs.enabled
		self:SetAlertSprite()
	end
end

--[[

function SteerClear:dumpKeys(o)
    local s = '['
	for k,v in pairs(o) do
	    s = s .. k .. '\n'
	end
	s = s .. ']'
	return s
end

function SteerClear:dump(o)
if type(o) == 'table' then
local s = '{ '
for k,v in pairs(o) do
if type(k) ~= 'number' then k = '"'..k..'"' end
s = s .. '['..k..'] = ' .. self:dump(v) .. ',\n'
end
return s .. '} '
else
return tostring(o)
end
end

--]]

function SteerClear:OnSteerClearOptions()
	if self.wndMain ~= nil then
	    self.wndMain:Show(true, true)
	end
end

-----------------------------------------------------------------------------------------------
-- SteerClearForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function SteerClear:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function SteerClear:OnCancel()
	self.wndMain:Close() -- hide the window
end

function SteerClear:OnBindButton(wndHandler, wndControl, eMouseButton)
	if self.currentBind == nil then
		self.wndMain:FindChild("Message"):SetText("Press button to bind.")
		wndControl:SetCheck(true)
		wndControl:SetFocus(true)
		self.currentBind = wndControl
	elseif wndControl == self.currentBind then
		self:BindKey("Mouse " .. (eMouseButton + 1), 2, eMouseButton, 0)
	end
end

function SteerClear:OnKeyDown(wndHandler, wndControl, strKeyName, nCode, eModifier)
		self:BindKey(strKeyName, 1, nCode, eModifier)
end

function SteerClear:BindKey(stringName, device, code, modifier)

	if self.currentBind ~= nil then
		self.wndMain:FindChild("Message"):SetText("")
		self.currentBind:SetText(stringName)
		self.currentBind:SetCheck(false)
		self.currentBind:SetFocus(false)
		
		local config = {}
		config.name = stringName
		config.input = {
			eDevice = device,
			eModifier = modifier,
			nCode = code
		}
		
		self.configs[self.currentBind:GetName()] = config
		
		self.currentBind = nil
	end
end

function SteerClear:OnKeyDownForSystem(key)
	if self.currentBind ~= nil then
		self.wndMain:FindChild("Message"):SetText("")
		self.currentBind:SetText(key)
		self.currentBind:SetCheck(false)
		self.currentBind:SetFocus(false)
		
		self.configs[self.currentBind:GetName()] = { 
			keybind = key
		}
		self.currentBind = nil
	end
end

-- Load and save.

function SteerClear:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end
	
	return self.configs
end

function SteerClear:OnRestore(eType, data)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	self.configs = data
end

-----------------------------------------------------------------------------------------------
-- SteerClear Instance
-----------------------------------------------------------------------------------------------
local SteerClearInst = SteerClear:new()
SteerClearInst:Init()
